/* Copyright 2009 Fog Creek Software, Inc. */

/* IPluginDatabaseExample version 1.0.0.0, uses data schema version 1. */

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Data;

/* FogBugz namespaces-- make sure you add the neccesary assembly references to
 * DLL files contained in C:\Program Files\FogBugz\website\bin\ */
using FogCreek.FogBugz;
using FogCreek.FogBugz.Plugins;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Entity;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz.UI;
using FogCreek.FogBugz.Database;

namespace IPluginDatabase_Example
{
    /* Class Declaration: Inherit from Plugin, implement IPluginDatabase, etc */
    public class IPluginDatabase_Example : Plugin, IPluginDatabase, IPluginPageDisplay,
        IPluginExtrasMenu
    {
        public const int pluginSchemaVersion = 1;
        
        /* Constructor: We'll just initialize the inherited Plugin class, which 
         * takes the passed instance of CPluginApi and sets its "api" member variable. */
        public IPluginDatabase_Example(CPluginApi api): base(api)
        {
        }

        #region IPluginDatabase Members

        public CTable[] DatabaseSchema()
        {
            /* Schema version 1 consists of a single "Kiwi" table */

            CTable kiwiTable = api.Database.NewTable(api.Database.PluginTableName("Kiwi"));
            kiwiTable.sDesc = "A table full of kiwis.";
            kiwiTable.AddAutoIncrementPrimaryKey("ixKiwi");
            kiwiTable.AddTextColumn("sKiwiFullName", "The full name of the kiwi.");

            /* In schema version 1, kiwis have an owner and a birth date is required */
            kiwiTable.AddIntColumn("ixOwner", false, "The key of the kiwi's owner used to look up the Person record.");
            kiwiTable.AddDateColumn("dtDateOfBirth", true, DateTime.Now, 
                "The kiwi's date of birth.");

            return new CTable[] { kiwiTable };
        }

        public int DatabaseSchemaVersion()
        {
            return pluginSchemaVersion;
        }

        /* This is the first version of the plugin, no need to handle an upgrade from an
         * earlier version. */
        
        public void DatabaseUpgradeAfter(int ixVersionFrom, int ixVersionTo, 
            CDatabaseUpgradeApi apiUpgrade)
        {
      
        }

        public void DatabaseUpgradeBefore(int ixVersionFrom, int ixVersionTo, 
            CDatabaseUpgradeApi apiUpgrade)
        {

        }

        #endregion

        #region IPluginPageDisplay Members

        public string PageDisplay()
        {
            string sHTML = "";
            sHTML += GetHeader();
            sHTML += GetForm();
            sHTML += GetKiwis();
            return sHTML;
        }

        public PermissionLevel PageVisibility()
        {
            /* We'll let all logged-in users with "normal" and "admin" permissions access
             * this page. */

            return PermissionLevel.Normal;
        }

        #endregion

        protected string GetHeader()
        {
            /* Generate an informational header, making use of the FogBugz UI library,
             * and grabbing this page's URL from the plugin API. */

            return String.Format(
                @"<p>{0}</p>
                  <p>
                      This page, generated by the <b>IPluginPageDisplay</b>
                      interface, demonstrates how the <b>IPluginDatabase</b> interface
                      can be used to create a custom database schema.
                  </p>     
                  <p>
                      The version of the plugin uses <b>version 1</b> of the database 
                      schema. This per-plugin version number is returned from 
                      <b>DatabaseSchemaVersion()</b>.
                  </p>",

                FogCreek.FogBugz.UI.PageDisplay.Headline("Adopt a Kiwi"));
        }

        protected string GetForm()
        {
            /* Generate a form that will post a value to the server and display
            * that value taken from the request object (if it exists).
            * 
            * IMPORTANT NOTE: The unique prefix of this plugin must be prepended to the
            * names of all form elements, but the values of those elements can then be
            * gathered from the request object without using the prefix. */

            return String.Format(
                @"<form action=""{0}"" method=""POST"">
                        <input type=""hidden"" name=""{1}actionToken"" value=""{2}"" />
                        <p>Add a Kiwi to your collection!</p>
                        <p>&nbsp;</p>
                        <p><b>Kiwi Name:</b> {3} {4}</p>
                </form>",

                HttpUtility.HtmlEncode(api.Url.PluginPageUrl()),
                api.PluginPrefix,
                api.Security.GetActionToken(),
                Forms.TextInput(api.PluginPrefix + "kiwiName", ""),
                Forms.SubmitButton("idSubmit", "Add Kiwi")
                );
        }

        protected void InsertKiwi(string sKiwiName)
        {
            /* Use the plugin api to get a CInsertQuery instance. Notice that
             * the table is given a unique prefix by FogBugz, and 
             * Database.PluginTableName returns us the full name. */
            
            CInsertQuery iq = api.Database.NewInsertQuery(
                api.Database.PluginTableName("Kiwi"));
            
            iq.InsertString("sKiwiFullName", sKiwiName);
            iq.InsertDate("dtDateOfBirth", api.TimeZone.UTCFromSTZ(DateTime.Now));
            iq.InsertInt("ixOwner", api.Person.GetCurrentPerson().ixPerson);       
            
            iq.Execute();
        }
        
        
        protected string GetKiwis()
        {
            /* No need to use the plugin prefix when checking the request object */
            if (api.Request[api.AddPluginPrefix("kiwiName")] != null)
            {
                if ((api.Request[api.AddPluginPrefix("actionToken")] == null) ||
                    !api.Security.ValidateActionToken(api.Request[api.AddPluginPrefix("actionToken")])
                    )
                {
                    /* if the action token doesn't match or is missing,
                     * display an error */
                    api.Notifications.AddError("action token invalid or missing");
                }
                else
                    InsertKiwi(api.Request[api.AddPluginPrefix("kiwiName")].ToString());
            }
            
            CSelectQuery sq = api.Database.NewSelectQuery(
                api.Database.PluginTableName("Kiwi"));
            sq.AddSelect("*");

            /* Iterate through the data set and display our kiwis in the table */
            
            DataSet ds = sq.GetDataSet();

            string sHTML = @"<table cellpadding=""2"" cellspacing=""0"">
                                <tr style=""background-color:#e9f0f6; font-weight:bold;"">
                                  <td width=""50""><b>Kiwi</b></td>
                                  <td width=""200""><b>Full Name</b></td>
                                  <td width=""200""><b>Date of Birth</b></td>
                                  <td width=""200""><b>Owner</b></td>
                                </tr>";

            if (ds.Tables[0] != null && ds.Tables[0].Rows.Count > 0)
            {
                for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                {
                    sHTML += "<tr><td>";
                    sHTML += ds.Tables[0].Rows[i]["ixKiwi"].ToString();
                    sHTML += "</td><td>";
                    sHTML += HttpUtility.HtmlEncode(
                        ds.Tables[0].Rows[i]["sKiwiFullName"].ToString());
                    sHTML += "</td><td>";
                    sHTML += api.TimeZone.CTZFromUTC(
                        Convert.ToDateTime(
                        ds.Tables[0].Rows[i]["dtDateOfBirth"])).ToString();
                    sHTML += "</td><td>";
                    /* fetching the person object for the owner makes sure
                     * we have the up-to-date full name and that we only
                     * display it if the currently logged-in person has
                     * permission to see it */
                    CPerson person = api.Person.GetPerson(
                        Convert.ToInt32(ds.Tables[0].Rows[i]["ixOwner"]));
                    sHTML += HttpUtility.HtmlEncode(person.sFullName);
                    sHTML += "</td></tr>";
                }
            }
            else
                sHTML += @"<tr>
                              <td colspan='3' style=""text-align:center; color:gray;"">
                                  <i>No Kiwis yet, add one now!</i>
                              </td>
                           </tr>";

            ds.Dispose();

            sHTML += "</table>";
            return sHTML;

        }

        #region IPluginExtrasMenu Members

        public CNavMenuLink[] ExtrasMenuLinks()
        {
            /* Create a link to this plugin's page in the "Extras" menu.
             * Because the page we link to is only visible to normal and
             * administrator users, we should only add an Extras menu link for
             * those users */
            if (api.Person.GetCurrentPerson().GetPermissionLevel() >
                PermissionLevel.Community)
                return new CNavMenuLink[] { new CNavMenuLink("Adopt a Kiwi!",
                    api.Url.PluginPageUrl()) };
            else
                return new CNavMenuLink[] { };
        }

        #endregion
    }
}